"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_statuses_1 = require("./constants/test-statuses");
const lodash_1 = require("lodash");
const errors_1 = require("./constants/errors");
const { NO_REF_IMAGE_ERROR } = errors_1.getCommonErrors();
const walk = (node, cb, fn = Array.prototype.some) => {
    return node.browsers && fn.call(node.browsers, cb) || node.children && fn.call(node.children, cb);
};
const hasFailedImages = (result) => {
    const { imagesInfo = [], status: resultStatus } = result;
    return imagesInfo.some(({ status }) => exports.isErroredStatus(status) ||
        exports.isFailStatus(status)) ||
        exports.isErroredStatus(resultStatus) ||
        exports.isFailStatus(resultStatus);
};
exports.hasNoRefImageErrors = ({ imagesInfo = [] }) => (Boolean(imagesInfo.filter((v) => lodash_1.get(v, 'reason.stack', '').startsWith(NO_REF_IMAGE_ERROR)).length));
exports.hasFails = (node) => {
    const { result } = node;
    const isFailed = result && hasFailedImages(result);
    return isFailed || walk(node, exports.hasFails);
};
exports.isAcceptable = ({ status, reason }) => {
    const stack = reason && reason.stack;
    return exports.isErroredStatus(status) && stack.startsWith(NO_REF_IMAGE_ERROR) || exports.isFailStatus(status);
};
exports.findNode = (node, suitePath) => {
    const path = suitePath.slice();
    if (!node.children) {
        const tree = {
            children: lodash_1.values(node),
            name: 'root',
        };
        return exports.findNode(tree, path);
    }
    const pathPart = path.shift();
    const child = lodash_1.find(node.children, { name: pathPart });
    if (!child) {
        return;
    }
    if (child.name === pathPart && !path.length) {
        return child;
    }
    return exports.findNode(child, path);
};
exports.setStatusForBranch = (nodes, suitePath, status) => {
    const node = exports.findNode(nodes, suitePath);
    if (!node) {
        return;
    }
    if ((exports.isSuccessStatus(status) || exports.isUpdatedStatus(status)) && exports.hasFails(node)) {
        return;
    }
    node.status = status;
    const path = suitePath.slice(0, -1);
    exports.setStatusForBranch(nodes, path, status);
};
exports.isSuccessStatus = (status) => status === test_statuses_1.SUCCESS;
exports.isFailStatus = (status) => status === test_statuses_1.FAIL;
exports.isIdleStatus = (status) => status === test_statuses_1.IDLE;
exports.isErroredStatus = (status) => status === test_statuses_1.ERROR;
exports.isSkippedStatus = (status) => status === test_statuses_1.SKIPPED;
exports.isUpdatedStatus = (status) => status === test_statuses_1.UPDATED;
