"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gemini_configparser_1 = __importDefault(require("gemini-configparser"));
const lodash_1 = __importDefault(require("lodash"));
const root = gemini_configparser_1.default.root;
const section = gemini_configparser_1.default.section;
const option = gemini_configparser_1.default.option;
const ENV_PREFIX = 'harry_reporter_';
const CLI_PREFIX = '--harry-reporter-';
const defaults_1 = require("./constants/defaults");
const assertType = (name, validationFn, type) => {
    return (value) => {
        if (!validationFn(value)) {
            throw new Error(`"${name}" option must be ${type}, but got ${typeof value}`);
        }
    };
};
const assertString = (name) => assertType(name, lodash_1.default.isString, 'string');
const assertBoolean = (name) => assertType(name, lodash_1.default.isBoolean, 'boolean');
const assertNumber = (name) => assertType(name, lodash_1.default.isNumber, 'number');
const getParser = () => {
    return root(section({
        baseHost: option({
            defaultValue: defaults_1.config.baseHost,
            validate: assertString('baseHost'),
        }),
        defaultView: option({
            defaultValue: defaults_1.config.defaultView,
            validate: assertString('defaultView'),
        }),
        enabled: option({
            defaultValue: true,
            parseCli: JSON.parse,
            parseEnv: JSON.parse,
            validate: assertBoolean('enabled'),
        }),
        gitUrl: option({
            defaultValue: defaults_1.config.gitUrl,
            validate: assertString('gitUrl'),
        }),
        lazyLoadOffset: option({
            defaultValue: defaults_1.config.lazyLoadOffset,
            parseEnv: JSON.parse,
            validate: assertNumber('lazyLoadOffset'),
        }),
        path: option({
            defaultValue: 'html-report',
            validate: assertString('path'),
        }),
        scaleImages: option({
            defaultValue: defaults_1.config.scaleImages,
            parseCli: JSON.parse,
            parseEnv: JSON.parse,
            validate: assertBoolean('scaleImages'),
        }),
    }), { envPrefix: ENV_PREFIX, cliPrefix: CLI_PREFIX });
};
exports.default = (options) => {
    const env = process.env;
    const argv = process.argv;
    return getParser()({ options, env, argv });
};
