"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const hermion_runner_1 = __importDefault(require("./hermion-runner"));
class App {
    constructor(paths, hermione, configs) {
        this.hermione = hermion_runner_1.default(paths, hermione, configs);
        this.browserConfigs = [];
        this.retryCache = {};
    }
    initialize() {
        return this.hermione.initialize();
    }
    finalize() {
        this.hermione.finalize();
    }
    run(tests) {
        return lodash_1.default.isEmpty(tests)
            ? this.hermione.run()
            : this.runWithoutRetries(tests);
    }
    updateReferenceImage(failedTests = []) {
        return this.hermione.updateReferenceImage(failedTests);
    }
    addClient(connection) {
        this.hermione.addClient(connection);
    }
    get data() {
        return this.hermione.tree;
    }
    runWithoutRetries(tests) {
        if (lodash_1.default.isEmpty(this.browserConfigs)) {
            this.browserConfigs = lodash_1.default.map(this.hermione.config.getBrowserIds(), id => this.hermione.config.forBrowser(id));
        }
        this.disableRetries();
        return this.hermione.run(tests)
            .finally(() => this.restoreRetries());
    }
    disableRetries() {
        this.browserConfigs.forEach((broConfig) => {
            this.retryCache[broConfig.id] = broConfig.retry;
            broConfig.retry = 0;
        });
    }
    restoreRetries() {
        this.browserConfigs.forEach((broConfig) => {
            broConfig.retry = this.retryCache[broConfig.id];
        });
    }
}
exports.default = App;
