"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_events_1 = __importDefault(require("../constants/client-events"));
const utils_1 = require("./utils");
const plugin_utils_1 = require("../../plugin-utils");
const test_statuses_1 = require("../../constants/test-statuses");
const reporter_helpers_1 = require("../../reporter-helpers");
exports.default = (hermione, reportBuilder, client, reportPath) => {
    function failHandler(testResult) {
        const formattedResult = reportBuilder.format(testResult);
        const actions = [reporter_helpers_1.saveTestImages(formattedResult, reportPath)];
        if (formattedResult.screenshot) {
            actions.push(reporter_helpers_1.saveBase64Screenshot(formattedResult, reportPath));
        }
        return Promise.all(actions);
    }
    hermione.on(hermione.events.SUITE_BEGIN, (suite) => {
        if (suite.pending) {
            return;
        }
        client.emit(client_events_1.default.BEGIN_SUITE, {
            name: suite.title,
            status: test_statuses_1.RUNNING,
            suitePath: plugin_utils_1.getSuitePath(suite),
        });
    });
    hermione.on(hermione.events.TEST_BEGIN, (data) => {
        const { browserId } = data;
        client.emit(client_events_1.default.BEGIN_STATE, {
            browserId,
            status: test_statuses_1.RUNNING,
            suitePath: plugin_utils_1.getSuitePath(data),
        });
    });
    hermione.on(hermione.events.TEST_PASS, (data) => {
        const formattedTest = reportBuilder.addSuccess(data);
        const testResult = utils_1.findTestResult(reportBuilder.getSuites(), formattedTest.prepareTestResult());
        reporter_helpers_1.saveTestImages(formattedTest, reportPath)
            .then(() => client.emit(client_events_1.default.TEST_RESULT, testResult));
    });
    hermione.on(hermione.events.TEST_FAIL, (data) => {
        const formattedResult = reportBuilder.format(data);
        formattedResult.hasDiff()
            ? reportBuilder.addFail(data)
            : reportBuilder.addError(data);
        const testResult = utils_1.findTestResult(reportBuilder.getSuites(), formattedResult.prepareTestResult());
        failHandler(data)
            .then(() => client.emit(client_events_1.default.TEST_RESULT, testResult));
    });
    hermione.on(hermione.events.RETRY, (data) => {
        reportBuilder.addRetry(data);
        failHandler(data);
    });
    hermione.on(hermione.events.RUNNER_END, (stats) => {
        return reportBuilder
            .setStats(stats)
            .save()
            .then(() => client.emit(client_events_1.default.END, { stats }));
    });
};
