"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const lodash_1 = __importDefault(require("lodash"));
const common_utils_1 = require("../../common-utils");
const formatTestHandler = (browser, test) => {
    const { suitePath, name } = test;
    return {
        browserId: browser.name,
        state: { name },
        suite: { path: suitePath.slice(0, -1) },
    };
};
exports.formatTests = (test) => {
    if (test.children) {
        return lodash_1.default.flatMap(test.children, child => exports.formatTests(child));
    }
    if (test.browserId) {
        test.browsers = lodash_1.default.filter(test.browsers, { name: test.browserId });
    }
    return lodash_1.default.flatMap(test.browsers, browser => formatTestHandler(browser, test));
};
exports.findTestResult = (suites = [], test) => {
    const { name, suitePath, browserId } = test;
    const nodeResult = common_utils_1.findNode(suites, suitePath);
    const browserResult = lodash_1.default.find(nodeResult.browsers, { name: browserId });
    return { name, suitePath, browserId, browserResult };
};
exports.formatId = (hash, browserId) => `${hash}/${browserId}`;
exports.getShortMD5 = (str) => {
    return crypto_1.default.createHash('md5').update(str, 'ascii').digest('hex').substr(0, 7);
};
exports.mkFullTitle = ({ suite, state }) => {
    // https://github.com/mochajs/mocha/blob/v2.4.5/lib/runnable.js#L165
    return `${suite.path.join(' ')} ${state.name}`;
};
