"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const body_parser_1 = __importDefault(require("body-parser"));
const express_1 = __importDefault(require("express"));
const path_1 = __importDefault(require("path"));
const signal_exit_1 = __importDefault(require("signal-exit"));
const server_utils_1 = require("../server-utils");
const app_1 = __importDefault(require("./app"));
const server_1 = require("./constants/server");
exports.default = ({ paths, hermione, guiApi, configs }) => {
    const { options, pluginConfig } = configs;
    const app = new app_1.default(paths, hermione, configs);
    const server = express_1.default();
    server.use(body_parser_1.default.json({ limit: server_1.MAX_REQUEST_SIZE }));
    guiApi.initServer(server);
    server.use(express_1.default.static(path_1.default.join(__dirname, '../static'), { index: 'gui.html' }));
    server.use(express_1.default.static(process.cwd()));
    server.use('/images', express_1.default.static(path_1.default.join(process.cwd(), pluginConfig.path, 'images')));
    server.get('/', (req, res) => res.sendFile(path_1.default.join(__dirname, '../static', 'gui.html')));
    server.get('/events', (req, res) => {
        res.writeHead(200, { 'Content-Type': 'text/event-stream' });
        app.addClient(res);
    });
    server.get('/init', (req, res) => {
        res.json(app.data);
    });
    server.post('/run', (req, res) => {
        app.run(req.body).catch((err) => {
            console.error('Error while trying to run tests', err);
        });
        res.sendStatus(200);
    });
    server.post('/update-reference', (req, res) => {
        app
            .updateReferenceImage(req.body)
            .then((updatedTests) => res.json(updatedTests))
            .catch(({ message }) => res.status(500).send({ error: message }));
    });
    signal_exit_1.default(() => {
        app.finalize();
        server_utils_1.logger.log('server shutting down');
    });
    return app
        .initialize()
        .then(() => {
        return bluebird_1.default.fromCallback((callback) => {
            server.listen(options.port, options.hostname, callback);
        });
    })
        .then(() => ({ url: `http://${options.hostname}:${options.port}` }));
};
