"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const plugin_1 = __importDefault(require("./plugin"));
const reporter_helpers_1 = require("./reporter-helpers");
module.exports = (hermione, opts) => {
    const plugin = new plugin_1.default(hermione, opts);
    if (!plugin.isEnabled()) {
        return;
    }
    plugin
        .addCliCommands()
        .init(prepareData, prepareImages);
};
const prepareData = (hermione, reportBuilder) => {
    const failHandler = (testResult) => {
        const formattedResult = reportBuilder.format(testResult);
        return formattedResult.hasDiff()
            ? reportBuilder.addFail(testResult)
            : reportBuilder.addError(testResult);
    };
    return new bluebird_1.default((resolve) => {
        // Test is skipped
        hermione.on(hermione.events.TEST_PENDING, (testResult) => reportBuilder.addSkipped(testResult));
        hermione.on(hermione.events.TEST_PASS, (testResult) => reportBuilder.addSuccess(testResult));
        hermione.on(hermione.events.TEST_FAIL, failHandler);
        hermione.on(hermione.events.RETRY, failHandler);
        // Will be triggered after test execution.
        // The handler accepts a stats of tests execution.
        hermione.on(hermione.events.RUNNER_END, (stats) => resolve(reportBuilder.setStats(stats)));
    });
};
const prepareImages = (hermione, pluginConfig, reportBuilder) => {
    const { path: reportPath } = pluginConfig;
    const failHandler = (testResult) => {
        const formattedResult = reportBuilder.format(testResult);
        const actions = [reporter_helpers_1.saveTestImages(formattedResult, reportPath)];
        if (formattedResult.screenshot) {
            actions.push(reporter_helpers_1.saveBase64Screenshot(formattedResult, reportPath));
        }
        return bluebird_1.default.all(actions);
    };
    return new bluebird_1.default((resolve, reject) => {
        let queue = bluebird_1.default.resolve();
        hermione.on(hermione.events.TEST_PASS, (testResult) => {
            queue = queue.then(() => reporter_helpers_1.saveTestImages(reportBuilder.format(testResult), reportPath));
        });
        hermione.on(hermione.events.RETRY, (testResult) => {
            queue = queue.then(() => failHandler(testResult));
        });
        hermione.on(hermione.events.TEST_FAIL, (testResult) => {
            queue = queue.then(() => failHandler(testResult));
        });
        hermione.on(hermione.events.RUNNER_END, () => queue.then(resolve, reject));
    });
};
