"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const bluebird_1 = __importDefault(require("bluebird"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const data_tree_1 = __importDefault(require("./data-tree"));
const server_utils_1 = require("../server-utils");
const fs = bluebird_1.default.promisifyAll(fs_extra_1.default);
const moveContentToReportDir = async ({ from, to }) => {
    const files = await fs.readdir(path_1.default.resolve(from));
    await bluebird_1.default.map(files, async (fileName) => {
        if (fileName === 'data.js') {
            return;
        }
        const srcFilePath = path_1.default.resolve(from, fileName);
        const destFilePath = path_1.default.resolve(to, fileName);
        await fs.copy(srcFilePath, destFilePath, { overwrite: true });
    });
};
class ReportBuilder {
    static create(srcPaths, destPath) {
        return new this(srcPaths, destPath);
    }
    constructor(srcPaths, destPath) {
        this.srcPaths = srcPaths;
        this.destPath = destPath;
    }
    async build() {
        await moveContentToReportDir({ from: this.srcPaths[0], to: this.destPath });
        const srcReportsData = this.loadReportsData();
        const dataTree = data_tree_1.default.create(srcReportsData[0], this.destPath);
        const srcDataCollection = lodash_1.default.zipObject(this.srcPaths.slice(1), srcReportsData.slice(1));
        const mergedData = await dataTree.mergeWith(srcDataCollection);
        await this.saveDataFile(mergedData);
    }
    loadReportsData() {
        return lodash_1.default(this.srcPaths)
            .map((reportPath) => {
            const srcDataPath = path_1.default.resolve(reportPath, 'data');
            try {
                return require(`../${srcDataPath}`);
            }
            catch (err) {
                server_utils_1.logger.warn(`Not found data file in passed source report path: ${reportPath}`);
                return { skips: [], suites: [] };
            }
        })
            .value();
    }
    async saveDataFile(data) {
        const formattedData = server_utils_1.prepareCommonJSData(data);
        const destDataPath = path_1.default.resolve(this.destPath, 'data.js');
        await fs.writeFile(destDataPath, formattedData);
    }
}
exports.default = ReportBuilder;
