"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const lodash_1 = __importDefault(require("lodash"));
const cli_commands_1 = __importDefault(require("./cli-commands"));
const utils = __importStar(require("./server-utils"));
const config_1 = __importDefault(require("./config"));
const report_builder_1 = __importDefault(require("./report-builder/report-builder"));
class Plugin {
    constructor(hermione, opts) {
        this.hermione = hermione;
        this.config = config_1.default(opts);
    }
    isEnabled() {
        return this.config.enabled;
    }
    addCliCommands() {
        lodash_1.default.values(cli_commands_1.default).forEach((command) => {
            this.hermione.on(this.hermione.events.CLI, (commander) => {
                const { default: cmd } = require(`./cli-commands/${command}`);
                cmd(commander, this.config, this.hermione);
                commander.prependListener(`command:${command}`, () => this.run = lodash_1.default.noop);
            });
        });
        return this;
    }
    init(prepareData, prepareImages) {
        this.hermione.on(this.hermione.events.INIT, () => this.run(prepareData, prepareImages));
        return this;
    }
    run(prepareData, prepareImages) {
        const reportBuilder = new report_builder_1.default(this.hermione, this.config);
        const generateReport = bluebird_1.default
            .all([
            prepareData(this.hermione, reportBuilder),
            prepareImages(this.hermione, this.config, reportBuilder),
        ])
            .then(() => reportBuilder.save())
            .then(utils.logPathToHtmlReport) // reportBuilder is undefined
            .catch(utils.logError);
        this.hermione.on(this.hermione.events.RUNNER_END, () => generateReport);
    }
}
exports.default = Plugin;
