"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const utils = __importStar(require("./server-utils"));
const saveAssertViewImages = (testResult, reportPath) => {
    return bluebird_1.default.map(testResult.assertViewResults, (assertResult) => {
        const { stateName } = assertResult;
        const actions = [];
        const src = assertResult.refImagePath || assertResult.refImg.path;
        if (!(assertResult instanceof Error)) {
            actions.push(utils.copyImageAsync(src, utils.getReferenceAbsolutePath(testResult, reportPath, stateName)));
        }
        if (testResult.isImageDiffError(assertResult)) {
            actions.push(exports.saveTestCurrentImage(testResult, reportPath, stateName), utils.saveDiff(assertResult, utils.getDiffAbsolutePath(testResult, reportPath, stateName)), utils.copyImageAsync(src, utils.getReferenceAbsolutePath(testResult, reportPath, stateName)));
        }
        if (testResult.isNoRefImageError(assertResult)) {
            actions.push(exports.saveTestCurrentImage(testResult, reportPath, stateName));
        }
        return bluebird_1.default.all(actions);
    });
};
exports.saveTestImages = (testResult, reportPath) => saveAssertViewImages(testResult, reportPath);
exports.saveTestCurrentImage = (testResult, reportPath, stateName) => {
    const curImg = testResult.getCurrImg(stateName);
    const src = (curImg.path || curImg) || testResult.getErrImg();
    return src
        ? utils.copyImageAsync(src, utils.getCurrentAbsolutePath(testResult, reportPath, stateName))
        : bluebird_1.default.resolve();
};
exports.updateReferenceImage = (testResult, reportPath, stateName) => {
    const src = testResult.actualPath
        ? path_1.default.resolve(reportPath, testResult.actualPath)
        : utils.getCurrentAbsolutePath(testResult, reportPath, stateName);
    return bluebird_1.default.all([
        utils.copyImageAsync(src, testResult.getImagePath(stateName)),
        utils.copyImageAsync(src, utils.getReferenceAbsolutePath(testResult, reportPath, stateName)),
    ]);
};
exports.saveBase64Screenshot = (testResult, reportPath) => {
    if (!testResult.screenshot) {
        utils.logger.warn('Cannot save screenshot on reject');
        return bluebird_1.default.resolve();
    }
    const destPath = utils.getCurrentAbsolutePath(testResult, reportPath);
    const screenshotBase64 = typeof testResult.screenshot === 'string' ?
        testResult.screenshot :
        testResult.screenshot.base64;
    return utils.makeDirFor(destPath)
        .then(() => fs_extra_1.default.writeFile(destPath, Buffer.from(screenshotBase64, 'base64'), 'base64'))
        .catch(err => console.log(err));
};
