"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const urijs_1 = __importDefault(require("urijs"));
const url_1 = __importDefault(require("url"));
const plugin_utils_1 = require("../plugin-utils");
class Suite {
    constructor(suite) {
        this.suite = suite;
    }
    get skipComment() {
        const skipComment = getSkipComment(this.suite);
        return this.wrapSkipComment(skipComment);
    }
    get fullName() {
        return this.suite.fullTitle();
    }
    get path() {
        return plugin_utils_1.getSuitePath(this.suite.parent);
    }
    get file() {
        return path_1.default.relative(process.cwd(), this.suite.file);
    }
    getUrl(opts) {
        const url = lodash_1.default.get(this, 'suite.meta.url', '');
        const baseHost = opts ? opts.baseHost : '';
        return this.configureUrl(url, baseHost);
    }
    get fullUrl() {
        const baseUrl = this.getUrl();
        return baseUrl
            ? url_1.default.parse(baseUrl).path
            : '';
    }
    wrapSkipComment(skipComment) {
        return skipComment ? wrapLinkByTag(skipComment) : 'Unknown reason';
    }
    configureUrl(url, baseHost) {
        return lodash_1.default.isEmpty(baseHost)
            ? url
            : urijs_1.default(baseHost).resource(url).href();
    }
}
exports.default = Suite;
const getSkipComment = (suite) => {
    return suite.skipReason || suite.parent && getSkipComment(suite.parent);
};
const wrapLinkByTag = (text) => {
    return text.replace(/https?:\/\/[^\s]*/g, (url) => {
        return `<a target="_blank" href="${url}">${url}</a>`;
    });
};
