"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const test_statuses_1 = require("../constants/test-statuses");
const plugin_utils_1 = require("../plugin-utils");
const server_utils_1 = require("../server-utils");
const suite_1 = __importDefault(require("../suite/suite"));
class TestResult {
    constructor(testResult, hermione) {
        this.testResult = testResult;
        this.hermione = hermione;
        this.errors = hermione.errors;
        this.suiteInternal = new suite_1.default(this.testResult);
    }
    get suite() {
        return this.suiteInternal;
    }
    get sessionId() {
        return this.testResult.sessionId || 'unknown session id';
    }
    get browserId() {
        return this.testResult.browserId;
    }
    get assertViewResults() {
        return this.testResult.assertViewResults || [];
    }
    get testBody() {
        return this.testResult.body;
    }
    get scenario() {
        const { hermioneCtx } = this.testResult;
        if (hermioneCtx && hermioneCtx.testpalm) {
            return hermioneCtx.testpalm.steps;
        }
        return null;
    }
    get imagesInfo() {
        return this.testResult.imagesInfo;
    }
    set imagesInfo(imagesInfo) {
        this.testResult.imagesInfo = imagesInfo;
    }
    getCurrImg(stateName) {
        const assertViewResult = lodash_1.default.find(this.assertViewResults, { stateName });
        return lodash_1.default.get(assertViewResult, 'currImg', lodash_1.default.get(assertViewResult, 'currentImagePath'));
    }
    getErrImg() {
        const path = typeof this.screenshot === 'string'
            ? this.screenshot
            : this.screenshot.path || undefined;
        return path;
    }
    hasDiff() {
        return this.assertViewResults.some((result) => this.isImageDiffError(result));
    }
    isImageDiffError(assertResult) {
        return assertResult instanceof this.errors.ImageDiffError;
    }
    isNoRefImageError(assertResult) {
        return assertResult instanceof this.errors.NoRefImageError;
    }
    getImagesInfo() {
        if (!lodash_1.default.isEmpty(this.imagesInfo)) {
            return this.imagesInfo;
        }
        this.imagesInfo = this.assertViewResults.map((assertResult) => {
            let status;
            let reason;
            if (!(assertResult instanceof Error)) {
                status = test_statuses_1.SUCCESS;
            }
            if (this.isImageDiffError(assertResult)) {
                status = test_statuses_1.FAIL;
            }
            if (this.isNoRefImageError(assertResult)) {
                status = test_statuses_1.ERROR;
                reason = lodash_1.default.pick(assertResult, ['message', 'stack']);
            }
            const { stateName, refImg } = assertResult;
            return lodash_1.default.extend({ stateName, refImg, status, reason }, server_utils_1.getPathsFor(status, this, stateName));
        });
        // common screenshot on test fail
        if (this.screenshot) {
            const errorImage = lodash_1.default.extend({ status: test_statuses_1.ERROR, reason: this.error }, server_utils_1.getPathsFor(test_statuses_1.ERROR, this));
            this.imagesInfo.push(errorImage);
        }
        return this.imagesInfo;
    }
    // hack which should be removed when html-reporter is able to show
    // all assert view fails for one test
    get firstAssertViewFail() {
        return lodash_1.default.find(this.testResult.assertViewResults, result => result instanceof Error);
    }
    get error() {
        return lodash_1.default.pick(this.testResult.err, ['message', 'stack', 'stateName']);
    }
    get imageDir() {
        return this.testResult.id();
    }
    get state() {
        return { name: this.testResult.title };
    }
    get attempt() {
        if (this.testResult.attempt >= 0) {
            return this.testResult.attempt;
        }
        const { retry } = this.hermione.config.forBrowser(this.testResult.browserId);
        return this.testResult.retriesLeft >= 0
            ? retry - this.testResult.retriesLeft - 1
            : retry;
    }
    // for correct determine image paths in gui
    set attempt(attemptNum) {
        this.testResult.attempt = attemptNum;
    }
    get screenshot() {
        return lodash_1.default.get(this.testResult, 'err.screenshot');
    }
    get assertViewState() {
        return this.firstAssertViewFail
            ? lodash_1.default.get(this.firstAssertViewFail, 'stateName')
            : lodash_1.default.get(this.testResult, 'err.stateName');
    }
    get description() {
        return this.testResult.description;
    }
    get meta() {
        return this.testResult.meta;
    }
    getImagePath(stateName) {
        return lodash_1.default.find(this.imagesInfo, { stateName }).imagePath || undefined;
    }
    prepareTestResult() {
        const { title: name, browserId } = this.testResult;
        const suitePath = plugin_utils_1.getSuitePath(this.testResult);
        return { name, suitePath, browserId };
    }
    get multipleTabs() {
        return true;
    }
}
exports.default = TestResult;
